# -*- coding: utf-8 -*-
"""
template handlers package.
"""

from pyrin.packaging.base import Package


class TemplateHandlersPackage(Package):
    """
    template handlers package class.
    """

    NAME = __name__
