# -*- coding: utf-8 -*-
"""
model package.
"""

from pyrin.packaging.base import Package


class ModelPackage(Package):
    """
    model package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'database.model.component'
