# -*- coding: utf-8 -*-
"""
request wrappers package.
"""

from pyrin.packaging.base import Package


class RequestWrappersPackage(Package):
    """
    request wrappers package class.
    """

    NAME = __name__
