# -*- coding: utf-8 -*-
"""
string normalizer package.
"""

from pyrin.packaging.base import Package


class StringNormalizerPackage(Package):
    """
    string normalizer package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'utilities.string.normalizer.component'
