# -*- coding: utf-8 -*-
"""
caching mixin package.
"""

from pyrin.packaging.base import Package


class CachingMixinPackage(Package):
    """
    caching mixin package class.
    """

    NAME = __name__
