# -*- coding: utf-8 -*-
"""
pyrin

an application framework to facilitate developing
small to large scale applications based on flask
which are more maintainable and extensible.
"""


__version__ = '0.4.16'
