# -*- coding: utf-8 -*-
"""
mimetype package.
"""

from pyrin.packaging.base import Package


class MIMETypePackage(Package):
    """
    mimetype package class.
    """

    NAME = __name__
    COMPONENT_NAME = 'processor.mimetype.component'
